<?php
/**
* @title			Minitek Content Utilities
* @copyright   		Copyright (C) 2011-2015 Minitek, All rights reserved.
* @license   		GNU General Public License version 3 or later.
* @author url   	http://www.minitek.gr/
* @developers   	Minitek.gr
*

*  --
*  Command-line script to create email queue in Minitek Content Utilities
*/
define('_JEXEC', 1);
define('DS', DIRECTORY_SEPARATOR);

if (file_exists(__DIR__ . '/defines.php'))
{
	include_once __DIR__ . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	$path = rtrim(__DIR__, DIRECTORY_SEPARATOR);
	$rpos = strrpos($path, DIRECTORY_SEPARATOR);
	$path = substr($path, 0, $rpos);
	define('JPATH_BASE', $path);
	require_once JPATH_BASE . '/includes/defines.php';
}

// Load the rest of the framework include files
if (file_exists(JPATH_LIBRARIES . '/import.legacy.php'))
{
	require_once JPATH_LIBRARIES . '/import.legacy.php';
}
else
{
	require_once JPATH_LIBRARIES . '/import.php';
}
require_once JPATH_LIBRARIES . '/cms.php';

// Load the JApplicationCli class
JLoader::import('joomla.application.cli');

/**
 * Minitek Content Utilities Email Queue Creation CLI app
 */
class MCUCreateEmailQueueApp extends JApplicationCli
{

	public function __construct(JInputCli $input = null, JRegistry $config = null, JDispatcher $dispatcher = null)
	{
		// Close the application if we are not executed from the command line, Akeeba style (allow for PHP CGI)
		if (array_key_exists('REQUEST_METHOD', $_SERVER))
		{
			die('You are not supposed to access this script from the web. You have to run it from the command line. If you don\'t understand what this means, you must not try to use this file before reading the documentation. Thank you.');
		}

		$cgiMode = false;

		if (!defined('STDOUT') || !defined('STDIN') || !isset($_SERVER['argv']))
		{
			$cgiMode = true;
		}

		// If a input object is given use it.
		if ($input instanceof JInput)
		{
			$this->input = $input;
		}
		// Create the input based on the application logic.
		else
		{
			if (class_exists('JInput'))
			{
				if ($cgiMode)
				{
					$query = "";
					if (!empty($_GET))
					{
						foreach ($_GET as $k => $v)
						{
							$query .= " $k";
							if ($v != "")
							{
								$query .= "=$v";
							}
						}
					}
					$query	 = ltrim($query);
					$argv	 = explode(' ', $query);
					$argc	 = count($argv);

					$_SERVER['argv'] = $argv;
				}

				$this->input = new JInputCLI();
			}
		}

		// If a config object is given use it.
		if ($config instanceof JRegistry)
		{
			$this->config = $config;
		}
		// Instantiate a new configuration object.
		else
		{
			$this->config = new JRegistry;
		}

		// If a dispatcher object is given use it.
		if ($dispatcher instanceof JDispatcher)
		{
			$this->dispatcher = $dispatcher;
		}
		// Create the dispatcher based on the application logic.
		else
		{
			$this->loadDispatcher();
		}

		// Load the configuration object.
		$this->loadConfiguration($this->fetchConfigurationData());

		// Set the execution datetime and timestamp;
		$this->set('execution.datetime', gmdate('Y-m-d H:i:s'));
		$this->set('execution.timestamp', time());

		// Set the current directory.
		$this->set('cwd', getcwd());
	}
		
	/**
	 * The main entry point of the application
	 */
	public function execute()
	{
		$this->out('Minitek Content Utilities -- Automatic Email Queue Creation script');
		$this->out('Copyright 2011-' . gmdate('Y') . ' Minitek.gr');
		$this->out(str_repeat('=', 79));
		
		JLoader::import('joomla.application.component.helper');
		JLoader::import('legacy.component.helper');
		JLoader::register('MinitekContentUtilitiesHelperUtilities', JPATH_ADMINISTRATOR. '/components/com_minitekcontentutilities/helpers/utilities.php');
		$utilities = new MinitekContentUtilitiesHelperUtilities();
		$app = JFactory::getApplication('site');
		$params = JComponentHelper::getParams('com_minitekcontentutilities');
		
		// Create Email Queue
		
		// Check if notification is published, current, x minutes old and email queue has not been created already
		$db = JFactory::getDBO();		
		$serverDate = JFactory::getDate()->format("Y-m-d H:i:s");
		$queue_delay = (int)$params->get('queue_delay', 60);
		
		$query = $db->getQuery(true);
		$query->select('*')
			->from('#__minitek_content_utilities_notifications AS n');
		
		$query->where(
			'n.state = 1 
			AND n.status IN (1,3,4)
			AND n.publish_up < DATE_SUB("'.$serverDate.'",INTERVAL '.$queue_delay.' MINUTE)'
		);
		$db->setQuery($query);
		
		$notifications = $db->loadObjectList();
		
		if ($notifications)
		{
			foreach ($notifications as $notification)
			{
				$notification_target_id = $notification->target_id;
				$notification_user_id = $notification->user_id;
				$notification_object_id = $notification->object_id;
				$notification_component_type_id = $notification->component_type_id;
				
				// Select Follows for notification target/user/object
				$query = $db->getQuery(true);
				$query->select('*')
					->from('#__minitek_content_utilities_subscriptions AS f');
				// select only if user has enabled email notifications
				if ($notification->type == '1')
				{
					$query->where(
						'f.state = 1 
						AND f.subscription_type_id = 1 
						AND f.component_type_id = '.$notification_component_type_id.' 
						AND	((f.object_type_id = 1 AND f.object_id = '.$notification_object_id.' AND f.notify = 1) 
						OR (f.object_type_id = 2 AND f.object_id = '.$notification_user_id.' AND f.notify = 1))'
					);	
				} else
				if ($notification->type == '2')
				{
					$query->where(
						'f.state = 1 
						AND f.subscription_type_id = 1 
						AND f.component_type_id = '.$notification_component_type_id.' 
						AND	((f.object_type_id = 1 AND f.object_id = '.$notification_object_id.' AND f.notify = 1) 
						OR (f.object_type_id = 2 AND f.object_id = '.$notification_user_id.' AND f.notify = 1)
						OR (f.object_type_id = 4 AND f.object_id = '.$notification_target_id.' AND f.notify = 1))'
					);	
				}
				
				$db->setQuery($query);
				$follows = $db->loadObjectList();
				
				if (count($follows))
				{
					foreach ($follows as $follow)
					{
						// Check if current user of follow, has blocked notification target/user/object
						$query = $db->getQuery(true);
						$query->select('*')
							->from('#__minitek_content_utilities_subscriptions AS b');
						$query->where(
							'b.state = 1 
							AND b.user_id = '.$follow->user_id.' 
							AND b.subscription_type_id = 2 
							AND b.component_type_id = '.$follow->component_type_id.' 
							AND ((b.object_type_id = 1 AND b.object_id = '.$notification_object_id.') 
							OR (b.object_type_id = 2 AND b.object_id = '.$notification_user_id.')
							OR (b.object_type_id = 4 AND b.object_id = '.$notification_target_id.'))'
						);
						
						$db->setQuery($query);
						$blocks = $db->loadObjectList();
						
						if (!$blocks)
						{
							$email_recipient = $follow->user_id;
							$email_created = $notification->publish_up;
							$email_status = '0';
							$email_notification_id = $notification->id;
							
							// Check if Email record exists - We don't want duplicate email records
							$query = $db->getQuery(true);
							$query->select('*')
								->from('#__minitek_content_utilities_email_queue AS e');
							$query->where(
								'e.recipient = '.$db->quote($email_recipient).' 
								AND e.notification_id = '.$db->quote($email_notification_id)
							);
							$db->setQuery($query);
							$emailExists = $db->loadObject();	
							
							if (!$emailExists)
							{
								// Get email template key
								$template_key = $utilities->getEmailTemplateKey($follow->object_type_id, $notification->type, $follow->component_type_id, $target_type = 'article');
								
								// Get recipient language
								$user = JFactory::getUser($email_recipient);
								$recipient_lang = $user->getParam('language', 'en-GB');
								
								// Select respective email template
								$email_template = $utilities->getEmailTemplate($template_key, $recipient_lang);
								
								// Create email queue subject/content
								$email_subject = $email_template->subject;
								$email_content = $email_template->content;
								
								// Get target title
								$target_title = $utilities->getEmailTargetTitle($follow->component_type_id, $notification->target_id);
								// Get target url
								$target_url = $utilities->getEmailTargetLink($follow->component_type_id, $notification->target_id);
								$target_url = str_replace("/cli/", "/", $target_url);
								
								// Get target author url
								$author_url = '';
								
								// Get target category name
								$category_name = $utilities->getObjectTitle($follow->component_type_id, $object_type_id = '1', $notification->object_id);
								// Get target category url
								$category_url = $utilities->getEmailObjectLink($follow->component_type_id, $object_type_id = '1', $notification->object_id);
								$category_url = str_replace("/cli/", "/", $category_url);
								
								$sitename = JURI::root();
								$sitename = str_replace("/cli/", "/", $sitename);
								
								// Replace placeholders
								$email_subject = str_ireplace(
									array (
										'[NOTIFICATION_DATE]',
										'[RECIPIENT_NAME]',
										'[TARGET_ID]',
										'[TARGET_TITLE]',
										'[TARGET_URL]',
										'[AUTHOR_NAME]',
										'[AUTHOR_URL]',
										'[CATEGORY_NAME]',
										'[CATEGORY_URL]',
										'[SITENAME]'
									),
									array (
										$notification->publish_up,
										$user->name,
										$notification->target_id,
										$target_title,
										$target_url,
										JFactory::getUser($notification->user_id)->name,
										$author_url,
										$category_name,
										$category_url,
										$sitename
									),
									$email_subject
								); 
								
								$email_content = str_ireplace(
									array (
										'[NOTIFICATION_DATE]',
										'[RECIPIENT_NAME]',
										'[TARGET_ID]',
										'[TARGET_TITLE]',
										'[TARGET_URL]',
										'[AUTHOR_NAME]',
										'[AUTHOR_URL]',
										'[CATEGORY_NAME]',
										'[CATEGORY_URL]',
										'[SITENAME]'
									),
									array (
										$notification->publish_up,
										$user->name,
										$notification->target_id,
										$target_title,
										$target_url,
										JFactory::getUser($notification->user_id)->name,
										$author_url,
										$category_name,
										$category_url,
										$sitename
									),
									$email_content
								);
								
								// Insert Email record
								$query = $db->getQuery(true);
								$columns = array(
									'recipient', 
									'subject', 
									'content', 
									'created', 
									'status', 
									'notification_id'
								);
								$values = array(
									$db->quote($email_recipient), 
									$db->quote($email_subject), 
									$db->quote($email_content), 
									$db->quote($email_created), 
									$db->quote($email_status), 
									$db->quote($email_notification_id)
								);
								$query
									->insert($db->quoteName('#__minitek_content_utilities_email_queue'))
									->columns($db->quoteName($columns))
									->values(implode(',', $values));
								 
								$db->setQuery($query);
								$db->execute();
							}
						}
					}
					
				}
				
				// Change status
				$query = $db->getQuery(true);	 
				if ($notification->status == '1') 
				{ 
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('5')
					);	 
				} else
				if ($notification->status == '4') 
				{
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('2')
					);	
				}
				$conditions = array(
					$db->quoteName('id') . ' = ' . $db->quote($notification->id)
				);		 
				$query->update($db->quoteName('#__minitek_content_utilities_notifications'))->set($fields)->where($conditions);
				$db->setQuery($query);
				$result = $db->execute();
			}
		}
		
		// End Create Email Queue

		$this->out('Minitek Content Utilities Email Queue Created');
		
	}

}

JApplicationCli::getInstance('MCUCreateEmailQueueApp')->execute();
