<?php
/**
* @title			Minitek Content Utilities
* @copyright   		Copyright (C) 2011-2015 Minitek, All rights reserved.
* @license   		GNU General Public License version 3 or later.
* @author url   	http://www.minitek.gr/
* @developers   	Minitek.gr
*

*  --
*  Command-line script to create updates in Minitek Content Utilities
*/
define('_JEXEC', 1);
define('DS', DIRECTORY_SEPARATOR);

if (file_exists(__DIR__ . '/defines.php'))
{
	include_once __DIR__ . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	$path = rtrim(__DIR__, DIRECTORY_SEPARATOR);
	$rpos = strrpos($path, DIRECTORY_SEPARATOR);
	$path = substr($path, 0, $rpos);
	define('JPATH_BASE', $path);
	require_once JPATH_BASE . '/includes/defines.php';
}

// Load the rest of the framework include files
if (file_exists(JPATH_LIBRARIES . '/import.legacy.php'))
{
	require_once JPATH_LIBRARIES . '/import.legacy.php';
}
else
{
	require_once JPATH_LIBRARIES . '/import.php';
}
require_once JPATH_LIBRARIES . '/cms.php';

// Load the JApplicationCli class
JLoader::import('joomla.application.cli');

/**
 * Minitek Content Utilities Updates Creation CLI app
 */
class MCUCreateUpdatesApp extends JApplicationCli
{

	public function __construct(JInputCli $input = null, JRegistry $config = null, JDispatcher $dispatcher = null)
	{
		// Close the application if we are not executed from the command line, Akeeba style (allow for PHP CGI)
		if (array_key_exists('REQUEST_METHOD', $_SERVER))
		{
			die('You are not supposed to access this script from the web. You have to run it from the command line. If you don\'t understand what this means, you must not try to use this file before reading the documentation. Thank you.');
		}

		$cgiMode = false;

		if (!defined('STDOUT') || !defined('STDIN') || !isset($_SERVER['argv']))
		{
			$cgiMode = true;
		}

		// If a input object is given use it.
		if ($input instanceof JInput)
		{
			$this->input = $input;
		}
		// Create the input based on the application logic.
		else
		{
			if (class_exists('JInput'))
			{
				if ($cgiMode)
				{
					$query = "";
					if (!empty($_GET))
					{
						foreach ($_GET as $k => $v)
						{
							$query .= " $k";
							if ($v != "")
							{
								$query .= "=$v";
							}
						}
					}
					$query	 = ltrim($query);
					$argv	 = explode(' ', $query);
					$argc	 = count($argv);

					$_SERVER['argv'] = $argv;
				}

				$this->input = new JInputCLI();
			}
		}

		// If a config object is given use it.
		if ($config instanceof JRegistry)
		{
			$this->config = $config;
		}
		// Instantiate a new configuration object.
		else
		{
			$this->config = new JRegistry;
		}

		// If a dispatcher object is given use it.
		if ($dispatcher instanceof JDispatcher)
		{
			$this->dispatcher = $dispatcher;
		}
		// Create the dispatcher based on the application logic.
		else
		{
			$this->loadDispatcher();
		}

		// Load the configuration object.
		$this->loadConfiguration($this->fetchConfigurationData());

		// Set the execution datetime and timestamp;
		$this->set('execution.datetime', gmdate('Y-m-d H:i:s'));
		$this->set('execution.timestamp', time());

		// Set the current directory.
		$this->set('cwd', getcwd());
	}

	/**
	 * The main entry point of the application
	 */
	public function execute()
	{
		$this->out('Minitek Content Utilities -- Automatic Updates Creation script');
		$this->out('Copyright 2011-' . gmdate('Y') . ' Minitek.gr');
		$this->out(str_repeat('=', 79));
		
		JLoader::import('joomla.application.component.helper');
		JLoader::import('legacy.component.helper');
		$params = JComponentHelper::getParams('com_minitekcontentutilities');
		
		// Create Updates
		$db = JFactory::getDBO();	
		$serverDate = JFactory::getDate()->format("Y-m-d H:i:s");		
		
		$query = $db->getQuery(true);
		$query->select('*')
			->from('#__minitek_content_utilities_notifications AS n');
		
		$query->where(
			'n.state = 1 
			AND n.status IN (1,3,5) 
			AND n.publish_up <= "'.$serverDate.'"'
		);
		$db->setQuery($query);
		
		$notifications = $db->loadObjectList();
		
		if ($notifications)
		{
			foreach ($notifications as $notification)
			{
				$notification_target_id = $notification->target_id;
				$notification_user_id = $notification->user_id;
				$notification_object_id = $notification->object_id;
				$notification_component_type_id = $notification->component_type_id;
				
				// Select Follows for notification target/user/object
				$query = $db->getQuery(true);
				$query->select('*')
					->from('#__minitek_content_utilities_subscriptions AS f');
					
				if ($notification->type == '1')
				{
					$query->where(
						'f.state = 1 
						AND f.subscription_type_id = 1 
						AND f.component_type_id = '.$notification_component_type_id.' 
						AND	((f.object_type_id = 1 AND f.object_id = '.$notification_object_id.') 
						OR (f.object_type_id = 2 AND f.object_id = '.$notification_user_id.')
						OR (f.object_type_id = 4 AND f.object_id = '.$notification_target_id.'))'
					);		
				} else
				if ($notification->type == '2')
				{
					// select only if user has enabled update notifications
					$query->where(
						'f.state = 1 
						AND f.subscription_type_id = 1 
						AND f.component_type_id = '.$notification_component_type_id.' 
						AND	((f.object_type_id = 1 AND f.object_id = '.$notification_object_id.' AND f.updates = 1) 
						OR (f.object_type_id = 2 AND f.object_id = '.$notification_user_id.' AND f.updates = 1)
						OR (f.object_type_id = 4 AND f.object_id = '.$notification_target_id.'))'
					);	
				} 
				
				$db->setQuery($query);
				$follows = $db->loadObjectList();
				
				if (count($follows))
				{
					foreach ($follows as $follow)
					{					
						// Check if current user of follow, has blocked notification target/user/object
						$query = $db->getQuery(true);
						$query->select('*')
							->from('#__minitek_content_utilities_subscriptions AS b');
						$query->where(
							'b.state = 1 
							AND b.user_id = '.$follow->user_id.' 
							AND b.subscription_type_id = 2 
							AND b.component_type_id = '.$follow->component_type_id.'
							AND ((b.object_type_id = 1 AND b.object_id = '.$notification_object_id.') 
							OR (b.object_type_id = 2 AND b.object_id = '.$notification_user_id.')
							OR (b.object_type_id = 4 AND b.object_id = '.$notification_target_id.'))'
						);
						$db->setQuery($query);
						$blocks = $db->loadObjectList();
						
						if (!$blocks)
						{
							$update_user_id = $follow->user_id;
							$update_author_id = $notification_user_id;
							$update_type = $notification->type;
							$update_target_id = $notification_target_id;
							$update_component_type_id = $notification->component_type_id;
							$update_object_id = $notification_object_id;
							$update_state = '1';
							$update_created = $notification->publish_up;
							
							// Check if Update record exists - We don't want duplicate records
							$query = $db->getQuery(true);
							$query->select('*')
								->from('#__minitek_content_utilities_updates AS u');
							$query->where(
								'u.user_id = '.$update_user_id.' 
								AND u.author_id = '.$update_author_id.' 
								AND u.type = '.$update_type.' 
								AND u.target_id = '.$update_target_id.' 
								AND u.component_type_id = '.$update_component_type_id.' 
								AND u.object_id = '.$update_object_id.' 
								AND u.state = '.$update_state.' 
								AND u.created = '.$db->quote($update_created)
							);
							$db->setQuery($query);
							$updateExists = $db->loadObject();	
							
							if (!$updateExists) 
							{
								// Insert update row 
								$query = $db->getQuery(true);
								$columns = array(
									'user_id', 
									'author_id', 
									'type', 
									'target_id', 
									'component_type_id', 
									'object_id', 
									'state', 
									'created'
								);
								$values = array(
									$db->quote($update_user_id), 
									$db->quote($update_author_id), 
									$db->quote($update_type), 
									$db->quote($update_target_id), 
									$db->quote($update_component_type_id), 
									$db->quote($update_object_id), 
									$db->quote($update_state), 
									$db->quote($update_created)
								);
								$query
									->insert($db->quoteName('#__minitek_content_utilities_updates'))
									->columns($db->quoteName($columns))
									->values(implode(',', $values));
								 
								$db->setQuery($query);
								$db->execute();	
							}
						} 
					}
				}
				
				// Change status
				$query = $db->getQuery(true);	
				if ($notification->status == '1') 
				{ 
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('4')
					);	 
				} else
				if ($notification->status == '5') 
				{
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('2')
					);	
				}
				$conditions = array(
					$db->quoteName('id') . ' = ' . $db->quote($notification->id)
				);		 
				$query->update($db->quoteName('#__minitek_content_utilities_notifications'))->set($fields)->where($conditions);
				$db->setQuery($query);
				$result = $db->execute();
			}
		}
		// End Create Updates

		$this->out('Minitek Content Utilities Updates Created');
		
	}

}

JApplicationCli::getInstance('MCUCreateUpdatesApp')->execute();
