<?php
/**
* @title			Minitek Content Utilities
* @copyright   		Copyright (C) 2011-2015 Minitek, All rights reserved.
* @license   		GNU General Public License version 3 or later.
* @author url   	http://www.minitek.gr/
* @developers   	Minitek.gr
*

*  --
*  Command-line script to send email notifications for Minitek Content Utilities
*/
define('_JEXEC', 1);
define('DS', DIRECTORY_SEPARATOR);

if (file_exists(__DIR__ . '/defines.php'))
{
	include_once __DIR__ . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	$path = rtrim(__DIR__, DIRECTORY_SEPARATOR);
	$rpos = strrpos($path, DIRECTORY_SEPARATOR);
	$path = substr($path, 0, $rpos);
	define('JPATH_BASE', $path);
	require_once JPATH_BASE . '/includes/defines.php';
}

// Load the rest of the framework include files
if (file_exists(JPATH_LIBRARIES . '/import.legacy.php'))
{
	require_once JPATH_LIBRARIES . '/import.legacy.php';
}
else
{
	require_once JPATH_LIBRARIES . '/import.php';
}
require_once JPATH_LIBRARIES . '/cms.php';

// Load the JApplicationCli class
JLoader::import('joomla.application.cli');

/**
 * Minitek Content Utilities Email Notifications CLI app
 */
class MCUSendEmailNotificationsApp extends JApplicationCli
{

	public function __construct(JInputCli $input = null, JRegistry $config = null, JDispatcher $dispatcher = null)
	{
		// Close the application if we are not executed from the command line, Akeeba style (allow for PHP CGI)
		if (array_key_exists('REQUEST_METHOD', $_SERVER))
		{
			die('You are not supposed to access this script from the web. You have to run it from the command line. If you don\'t understand what this means, you must not try to use this file before reading the documentation. Thank you.');
		}

		$cgiMode = false;

		if (!defined('STDOUT') || !defined('STDIN') || !isset($_SERVER['argv']))
		{
			$cgiMode = true;
		}

		// If a input object is given use it.
		if ($input instanceof JInput)
		{
			$this->input = $input;
		}
		// Create the input based on the application logic.
		else
		{
			if (class_exists('JInput'))
			{
				if ($cgiMode)
				{
					$query = "";
					if (!empty($_GET))
					{
						foreach ($_GET as $k => $v)
						{
							$query .= " $k";
							if ($v != "")
							{
								$query .= "=$v";
							}
						}
					}
					$query	 = ltrim($query);
					$argv	 = explode(' ', $query);
					$argc	 = count($argv);

					$_SERVER['argv'] = $argv;
				}

				$this->input = new JInputCLI();
			}
		}

		// If a config object is given use it.
		if ($config instanceof JRegistry)
		{
			$this->config = $config;
		}
		// Instantiate a new configuration object.
		else
		{
			$this->config = new JRegistry;
		}

		// If a dispatcher object is given use it.
		if ($dispatcher instanceof JDispatcher)
		{
			$this->dispatcher = $dispatcher;
		}
		// Create the dispatcher based on the application logic.
		else
		{
			$this->loadDispatcher();
		}

		// Load the configuration object.
		$this->loadConfiguration($this->fetchConfigurationData());

		// Set the execution datetime and timestamp;
		$this->set('execution.datetime', gmdate('Y-m-d H:i:s'));
		$this->set('execution.timestamp', time());

		// Set the current directory.
		$this->set('cwd', getcwd());
	}

	/**
	 * The main entry point of the application
	 */
	public function execute()
	{
		$this->out('Minitek Content Utilities -- Automatic Email Notifications script');
		$this->out('Copyright 2011-' . gmdate('Y') . ' Minitek.gr');
		$this->out(str_repeat('=', 79));
		
		JLoader::import('joomla.application.component.helper');
		JLoader::import('legacy.component.helper');
		$params = JComponentHelper::getParams('com_minitekcontentutilities');
		$emails_per_batch = (int)$params->get('emails_per_batch', 50);
		
		// Send Email Queue
		
		// Select all pending / failed emails from queue / LIMIT emails_per_batch
		$db = JFactory::getDBO();	
		$query = $db->getQuery(true);
		$query->select('*')
			->from('#__minitek_content_utilities_email_queue AS eq');
		
		$query->where(
			'eq.status IN (0,2)'
		);
		$query->order('id ASC');
		$query->setLimit($emails_per_batch);
		
		$db->setQuery($query);
		
		$emailqueue = $db->loadObjectList();
		
		if ($emailqueue)
		{
			foreach ($emailqueue as $email)
			{
				$user = JFactory::getUser($email->recipient);
				$recipient = $user->email;
				
				$mailer = JFactory::getMailer();
				$config = JFactory::getConfig();
				$sender = array( 
					$config->get( 'mailfrom' ),
					$config->get( 'fromname' ) );
				
				$mailer->setSender($sender);
				$mailer->addRecipient($recipient);				
				$mailer->setSubject($email->subject);
				$mailer->isHTML(true);
				$mailer->Encoding = 'base64';
				$mailer->setBody($email->content);
				
				// Send Email
				$send = $mailer->Send();
				if ( $send !== true ) 
				{
					$this->out('Error sending email: ' . $send->__toString());
					
					// Change status
					$query = $db->getQuery(true);	 
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('2')
					);	
					$conditions = array(
						$db->quoteName('id') . ' = ' . $db->quote($email->id)
					);		 
					$query->update($db->quoteName('#__minitek_content_utilities_email_queue'))->set($fields)->where($conditions);
					$db->setQuery($query);
					$result = $db->execute();
				} 
				else 
				{
					$this->out('Mail sent');
					
					// Change status
					$query = $db->getQuery(true);	 
					$fields = array(
						$db->quoteName('status') . ' = ' . $db->quote('1')
					);	
					$conditions = array(
						$db->quoteName('id') . ' = ' . $db->quote($email->id)
					);		 
					$query->update($db->quoteName('#__minitek_content_utilities_email_queue'))->set($fields)->where($conditions);
					$db->setQuery($query);
					$result = $db->execute();
				}
				
			}
		}
		// End Send Email Queue

		$this->out('Minitek Content Utilities Emails Sent');
		
	}

}

JApplicationCli::getInstance('MCUSendEmailNotificationsApp')->execute();
